/*
NAME: 
DESCRIPTION: ;
PARAMETER:
    [
		{
			name : 'cutting_marign',
			title : '切除profile外数值',
			type : 'LineEdit',
			property : {tool_tip : '切除profile外数值,默认是0.2mm'},
		},
		{
            name : 'auto_save',
			title : '自动保存',
            type : 'RadioBox',
            property : {
				item_list:[
					{name:'yes',text:'YES'},
					{name:'no',text:'NO'},
				],
				tool_tip:'是否自动保存料号开关'
			}
        }
	]
	
 VERSION_HISTORY:
	V1.00 2020-04-20 Scott Sun
	    1.新版本
		
 HELP:
 	<html><body bgcolor="#DDECFE">
		<font size="3" color="#003DB2"><p>功能简介</p></font>
		<p> 删除板外 </p>
		<br>
		<font size="3" color="#003DB2"><p>参数配置</p></font>
		<p> 切除值 </p>
		<br>
		<font size="3" color="#003DB2"><p>注意事项</p></font>
		<p> 无 </p>
		<br>
	</body></html>	
*/
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
console.log("==============================>删除板外");
// 引入模块 包
var $ = require('topcam.scriptfunc').argv();
var fs = require('fs');
var _ = require('lodash');
var database = require("topsin.database");
database.addConnection($.conf.database_conf, "DFM");
var QDfm = database.query("DFM");
if ($.conf.product_type == "aimdfm") {
	QDfm.updateRow({
		table: "pdm_aimdfm_task",
		data: {
			current_process_title: $.process_title
		},
		where: { id: $.task_id }
	});
}
var GEN = $.gen;
var Job = $.job_name;
var db = $.db
var Status = 'ok';
var resultData = [];
var PAR = {};
if ($.hasOwnProperty('script_parameter')){
	PAR = JSON.parse($.script_parameter);
}

try {
	var par = PAR;
	var default_par = {
		cutting_marign: "0.2mm",
		auto_save: "No",
	}
	for(var key in default_par){  // 设置默认属性
		if (!par.hasOwnProperty(key) || par[key] == ""){
			par[key] = default_par[key]
		}
	}
	par.cutting_marign = parseFloat(par.cutting_marign)
	if(!/^(([^0][0-9]+|0)\.([0-9]{1,2})$)|^([^0][0-9]+|0)$/ig.test(par.cutting_marign)){throw "cutting_marign failed"}
	var cutting_marign_symbol = "r" + String(par.cutting_marign * 1000)
	if(_.isEmpty(Job)){throw "参数Job不存在"}
	var job = Job.toLowerCase()
	// 检查料号是否存在
	if(!GEN.isJobExists({job:job})){throw "料号已经:"+job+"不存在"}
	// 检查料号是否被打开
	if(!GEN.isJobOpen({job:job})){ GEN.openJob({job:job}) }
	// 检查料号是否能够check out
	if(GEN.checkInout({job:job,mode:"test"}) != 0){ throw "the job check" }
	GEN.checkInout({job:job,mode:"out"});
	// 通过matrix获取board层
	var matrix = GEN.getMatrix({job:job})
	var board_layers = Object.keys(matrix).reduce(function(a,b){
		if(matrix[b].context == "board"){
			a.push(b)
		}
		return a
	},[])
	
	// 影响board层
	var steplist = GEN.getStepList({job:job})
	steplist.forEach(function(step){
		GEN.openStep({job:job,name:step})
		GEN.clearLayers()
		GEN.affectedLayer({affected:"no", mode: "all"})
		GEN.affectedLayer({affected:"yes",layer:board_layers})
		// 切除profile以外0.2mm（cutting_marign）物件
		GEN.selectByFilter({
			include_syms:cutting_marign_symbol,
			profile: "out"
		})
		if(GEN.getSelectCount()> 0){
			GEN.selDelete()   
		}
		GEN.affectedLayer({affected:'no',mode:'all'})
		GEN.closeStep()
	})

	// 保存 
	if(/yes/ig.test(par.auto_save)){
		GEN.checkInout({job:job,mode:"out"})  // 结束保存料号 关闭料号
		GEN.saveJob({ job: job });
		GEN.checkInout({job:job,mode:"in"})
		GEN.closeJob({job:job})
	} else {
		GEN.checkInout({job:job,mode:"in"})
	}
    QDfm.updateRow({
		table: "pdm_aimdfm_task",
		data: {
			progress: 33.33
		},
		where: { id: $.task_id }
	});
	if (GEN.hasError()) {
		Status = 'error';
		resultData.push({ type: "error", title: "GEN错误!", detail: [{ desc: _.join(GEN.STATUS, "\n") }] });
		return {
			status: Status,
			result_data: resultData
		};
	} else {
		resultData.push({ type: "info", title: "操作完成, 请注意检查!" });
		return {
			status: Status,
			result_data: resultData
		};
	}
} catch (e) {
    Status = 'error';
    resultData.push({type: "error", title: "脚本执行出错!", detail: [{desc: _.toString(e)}]});
    return {status: Status, result_data: resultData};
}